/*----------------------------------------------------------------------------*
 *  Copyright (c) 2013        Southeastern Universities Research Association, *
 *                            Thomas Jefferson National Accelerator Facility  *
 *                                                                            *
 *    This software was developed under a United States Government license    *
 *    described in the NOTICE file included as part of this distribution.     *
 *                                                                            *
 *    Authors: Bryan Moffit                                                   *
 *             moffit@jlab.org                   Jefferson Lab, MS-12B3       *
 *             Phone: (757) 269-5660             12000 Jefferson Ave.         *
 *             Fax:   (757) 269-5800             Newport News, VA 23606       *
 *                                                                            *
 *----------------------------------------------------------------------------*
 *
 * Description:
 *     Firmware update for the Crate Trigger Processor (CTP) module.
 *
 *----------------------------------------------------------------------------*/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#ifdef VXWORKS
#include "vxCompat.h"
#else
#include "jvme.h"
#endif
#include "tiLib.h"
#include "ctpLib.h"


char *programName;
enum ifpga {U1, U3, U24, NFPGA};
const char *fpga_names[NFPGA] = {
    "U1",
    "U3",
    "U24"
};

#ifndef VXWORKS
static void ctpFirmwareUsage();
#endif

int
#ifdef VXWORKS
ctpFirmwareUpdate(unsigned int arg_vmeAddr, char *arg_filename, int arg_ifpga)
#else
main(int argc, char *argv[])
#endif
{
  int stat;
  int ifpga;
  char *filename;
  int inputchar=10;
  unsigned int vme_addr=0;
  int reboot=1;
#ifdef SKIPTHIS
  int nbytes=0; int nsectors=0;
#endif
  
  printf("\nCTP firmware update via VME\n");
  printf("----------------------------\n");

#ifdef VXWORKS
  programName = __FUNCTION__;

  vme_addr = arg_vmeAddr;
  filename = arg_filename;
  ifpga    = arg_ifpga;
#else
  programName = argv[0];

  if(argc<4)
    {
      printf(" ERROR: Must specify three arguments\n");
      ctpFirmwareUsage();
      return(-1);
    }
  else
    {
      vme_addr = (unsigned int) strtoll(argv[1],NULL,16)&0xffffffff;
      filename = argv[2];
      ifpga = (unsigned int) strtoll(argv[3],NULL,10);
    }

  vmeSetQuietFlag(1);
  stat = vmeOpenDefaultWindows();
  if(stat != OK)
    goto CLOSE;
#endif

#ifdef SKIPTHIS
  printf("Enter number of bytes to use from firmware file (0 for all): ");
  scanf("%d",&nbytes);
  if(nbytes>0)
    ctpSetFWSize(nbytes);
  else
    printf(" ..will use full firmware size\n");

  printf("Enter number of sectors to erase (0 for all): ");
  scanf("%d",&nsectors);
  if(nsectors>0)
    ctpSetNSectorErase(nsectors);
  else
    printf(" ..will erase all sectors\n");
#endif

  stat = tiInit(vme_addr,0,1);
  if(stat != OK)
    {
      printf("\n");
      printf("*** Failed to initialize TI ***\n");
      goto CLOSE;
    }

  stat = ctpInit(CTP_INIT_IGNORE_VERSION);
  if(stat != OK)
    {
      printf("\n");
      printf("*** Failed to initialize CTP ***\n");
      goto CLOSE;
    }

  char ctpSN[20];
  int  fwversion=0;

  stat = ctpGetSerialNumber((char **)&ctpSN);
  if(stat>0)
    printf(" CTP Serial Number : %s\n",ctpSN);
  else
    printf("ERROR getting serial number (returned %d)\n",stat);

  fwversion = ctpGetFirmwareVersion(ifpga);
  printf(" %s: Firmware Version = 0x%04x\n",
	 fpga_names[ifpga],fwversion);
  printf(" Update with file: %s\n\n",filename);

  printf("Proceed with the CTP Firmware update?\n");
 REPEAT:
  printf(" (y/n): ");
  inputchar = getchar();
  
  if((inputchar == 'n') || (inputchar == 'N'))
    {
      printf("--- Exiting without update ---\n");
      goto CLOSE;
    }
  else if((inputchar == 'y') || (inputchar == 'Y'))
    {
      printf("--- Continuing with update ---\n");
    }
  else
    {
      goto REPEAT;
    }
  
  stat = ctpFirmwareUpload(filename,ifpga,reboot);
  if(stat != OK)
    {
      printf("\n");
      printf("*** CTP Firmware Update Failed ***\n");
      goto CLOSE;
    }
  else
    {
      printf("\n");
      printf("--- CTP Firmware Update Succeeded ---\n");
    }
 CLOSE:

#ifndef VXWORKS
  vmeCloseDefaultWindows();
#endif
  printf("\n");

  return OK;
}

#ifndef VXWORKS
static void
ctpFirmwareUsage()
{
  int ifpga=0;
  printf("\n");
  printf("%s <TI VME Address (A24)> <firmware mcs file> <FPGA number>\n",programName);
  printf("     where FPGA number :");
  for(ifpga=U1; ifpga<NFPGA; ifpga++)
    {
      if(ifpga!=U1)
	printf("                        ");
      printf(" %d - %s\n",ifpga,fpga_names[ifpga]);
    }
  printf("\n");

}
#endif
