/************************************************************
 * 
 * flexio_rol.c - "Primary" Readout list routines using
 *                a flexio as the trigger interface
 *
 * Usage:
 *
 *    #include "flexio_list.c"
 *
 *  then define the following routines:
 * 
 *    void rocDownload(); 
 *    void rocPrestart(); 
 *    void rocGo(); 
 *    void rocEnd(); 
 *
 */

#define ROL_NAME__ "GEN_USER"
#ifndef MAX_EVENT_LENGTH
#define MAX_EVENT_LENGTH 1024*8
#endif
#ifndef MAX_EVENT_POOL
#define MAX_EVENT_POOL   500
#endif
/* POLLING_MODE */
#define POLLING___
#define POLLING_MODE
#define GEN_MODE
/* INIT_NAME should be defined by roc_### (maybe at compilation time - check Makefile-rol) */
#ifndef INIT_NAME
#define INIT_NAME gen_list__init
#endif
#include <rol.h>
#include <GEN_source.h>
#include "jvme.h"

unsigned long long time_start, time_end;

extern int bigendian_out;
extern unsigned int tirNeedAck;

extern volatile int flexioDoAck[2];
extern volatile unsigned int flexioNeedAck[2];

void rocDownload();
void rocPrestart();
void rocGo();
void rocEnd();
int  getOutQueueCount();


static void __download()
{
  daLogMsg("INFO","Readout list compiled %s", DAYTIME);
#ifdef POLLING___
  rol->poll = 1;
#endif
  *(rol->async_roc) = 0; /* Normal ROC */
  {  /* begin user */
    bigendian_out=1;
    {/* inline c-code */
 
      /* Open the default VME windows */
      vmeOpenDefaultWindows();
      /* Disable IRQ for VME Bus Errors */
      vmeDisableBERRIrq();

      rocDownload();
 
    }/*end inline c-code */
    daLogMsg("INFO","User Download Executed");

  }  /* end user */
} /*end download */     

static void __prestart()
{
  CTRIGINIT;
  *(rol->nevents) = 0;
  {  /* begin user */
    unsigned long jj, adc_id;
    daLogMsg("INFO","Entering User Prestart");

    GEN_INIT;
    CTRIGRSS(GEN,1,usrtrig,usrtrig_done);
    CRTTYPE(1,GEN,1);
    rocPrestart();
    daLogMsg("INFO","User Prestart Executed");

  }  /* end user */
  if (__the_event__) WRITE_EVENT_;
  *(rol->nevents) = 0;
  rol->recNb = 0;
} /*end prestart */     

static void __end()
{
  {  /* begin user */
    int ii, ievt, rem_count;
    int len, type, lock_key;
    DMANODE *outEvent;
    int oldnumber;

    rocEnd();
    CDODISABLE(GEN,1,0);
    {/* inline c-code */
 
      /* we need to make sure all events taken by the
	 VME are collected from the vmeOUT queue */
#define HAVE_SECONDARY_ROL
#ifdef HAVE_SECONDARY_ROL
      rem_count = getOutQueueCount();
      if (rem_count > 0) {
	printf("gen_list End: %d events left on vmeOUT queue (will now de-queue them)\n",rem_count);
	/* This wont work without a secondary readout list (will crash EB or hang the ROC) */
	for(ii=0;ii<rem_count;ii++) {
	  __poll();
	}
	dmaPStatsAll();

      }else{
	printf("gen_list End: vmeOUT queue Empty\n");
      }
 #endif
    }/*end inline c-code */
    daLogMsg("INFO","User End Executed");


  }  /* end user */
  if (__the_event__) WRITE_EVENT_;
} /* end end block */

static void __pause()
{
  {  /* begin user */
    CDODISABLE(GEN,1,0);
    daLogMsg("INFO","User Pause Executed");

  }  /* end user */
  if (__the_event__) WRITE_EVENT_;
} /*end pause */
static void __go()
{

  {  /* begin user */
    daLogMsg("INFO","Entering User Go");

    CDOENABLE(GEN,1,1);
    rocGo();
  }  /* end user */
  if (__the_event__) WRITE_EVENT_;


}

extern void flexioIntAck(unsigned int card);

void usrtrig(unsigned long EVTYPE,unsigned long EVSOURCE)
{
  long EVENT_LENGTH;

  int ii, len, type, lock_key;
  unsigned int thisdata[MAX_EVENT_LENGTH/4];
  DMANODE *outEvent;
 
  outEvent = dmaPGetItem(vmeOUT);
  if(outEvent) 
    {
      len = outEvent->length;
      type = outEvent->type;
      CEOPEN(type, BT_UI4);
	  
      if(rol->dabufp != NULL) 
	{
	  memcpy(rol->dabufp,outEvent->data,len*sizeof(unsigned int));
	  rol->dabufp += len;
	} 
      else 
	{
	  printf("gen_list: ERROR rol->dabufp is NULL -- Event lost\n");
	}

      /* Free up the trigger after freeing up the buffer - do it
	 this way to prevent triggers from coming in after freeing
	 the buffer and before the Ack */
      if(flexioNeedAck[0] || flexioNeedAck[1]) 
	{
	  outEvent->part->free_cmd = *flexioIntAck;
	  if(flexioNeedAck[0] && flexioNeedAck[1])
	    {
	      outEvent->part->clientData = 2;
	    }
	  else if(flexioNeedAck[0]) 
	    {
	      outEvent->part->clientData = 0;
	    }
	  else if(flexioNeedAck[1])
	    {
	      outEvent->part->clientData = 1;
	    }
	}
      else
	{
	  outEvent->part->free_cmd = NULL;
	}

      CECLOSE;
      dmaPFreeItem(outEvent); /* flexioIntAck performed in here, if flexioNeedAck */
    }
  else
    {
      logMsg("Error: no Event in vmeOUT queue\n",0,0,0,0,0,0);
    }

 
} /*end trigger */

void usrtrig_done()
{
  {  /* begin user */
  }  /* end user */
} /*end done */

void __done()
{
  poolEmpty = 0; /* global Done, Buffers have been freed */
  {  /* begin user */
  }  /* end user */
} /*end done */

static void __status()
{
  {  /* begin user */
  }  /* end user */
} /* end status */

int
getOutQueueCount()
{
  if(vmeOUT) 
    return(vmeOUT->list.c);
  else
    return(0);
}
