/*
 * File:
 *    fadcLibTest2.c
 *
 * Description:
 *    Test JLab Flash ADC with GEFANUC Linux Driver
 *    and fadc library
 *
 *
 */


#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include "gef/gefcmn_vme.h"
#include "jlabgef.h"
#include "tirLib.h"
#include "fadcLib.h"


/* VME Window Prototypes */
GEF_STATUS vmeOpenDefaultWindows();
GEF_STATUS vmeCloseDefaultWindows();

void faGClearError();

extern int fadcA32Base;
extern int fadcA32Offset;


/* access tirLib global variables */
extern unsigned int tirIntCount;
int FA_SLOT;

#define FADC_ADDR 0xee0000

/* Interrupt Service routine */
void
mytirISR(int arg)
{
  unsigned short reg;
  int ii,stat;

  reg = tirReadData();
  if(tirIntCount%1000 == 0) {
    printf("Got %d interrupts (Data = 0x%x)\n",tirIntCount,reg);
    for(ii=0;ii<100;ii++) {
      stat = faDready(FA_SLOT);
      if (stat>0) {
	break;
      }
    }
    if(stat>0) {
      faPrintBlock(FA_SLOT,1);
    } else {
      printf (" no events\n");
    }
    
  }
  tirIntOutput(1);
  tirIntOutput(0);
}

int 
main(int argc, char *argv[]) 
{
  int stat;
  extern int fadcID[20];
  unsigned short iflag;

  printf("\nJLAB fadc Lib Tests\n");
  printf("----------------------------\n");
  
  if(vmeOpenDefaultWindows() != GEF_STATUS_SUCCESS) {
    goto CLOSE;
  }
  
  tirIntInit(0,0,1);
  tirIntStatus(1);
  
  iflag = 0xea00; // SDC Board address
  iflag |= 1<<0;  // Front panel sync-reset
  iflag |= 1<<1;  // Front Panel Input trigger source
/*   iflag |= 0<<4;  // Internal 250MHz Clock source */
  iflag |= 1<<4;  // Front Panel 250MHz Clock source
  printf("iflag = 0x%x\n",iflag);

  fadcA32Base = 0x08000000;
  
  faInit(FADC_ADDR,0x0,1,iflag);
  faGClearError();
  FA_SLOT = fadcID[0];

  /*      Setup FADC Programming */
  faSetBlockLevel(FA_SLOT,1);
  /*  for Block Reads */
  faEnableBusError(FA_SLOT);
  /*  for Single Cycle Reads */
  /*   faDisableBusError(FA_SLOT); */
  
  /*  Set All channel thresholds to 0 */
  faSetThreshold(FA_SLOT,0,0xffff);
  
    
  /*  Setup option 1 processing - RAW Window Data     <-- */
  /*        option 2            - RAW Pulse Data */
  /*        option 3            - Integral Pulse Data */
  /*  Setup 200 nsec latency (PL  = 50)  */
  /*  Setup  80 nsec Window  (PTW = 20) */
  /*  Setup Pulse widths of 36ns (NSB(3)+NSA(6) = 9)  */
  /*  Setup up to 1 pulse processed */
  /*  Setup for both ADC banks(0 - all channels 0-15) */
  faSetProcMode(FA_SLOT,1,50,20,3,6,1,0);

  faStatus(0,0);

  faSDC_Status(0);

  faSDC_Sync();
    
  stat = tirIntConnect(TIR_INT_VEC, mytirISR, 0, NULL);
  if (stat != OK) {
    printf("ERROR: tirIntConnect failed: \n");
    goto CLOSE;
  } else {
    printf("INFO: Attached TIR Interrupt\n");
  }





/*  Enable FADC */
  faEnable(FA_SLOT,0,0);

  taskDelay(1);
  
/*  Send Sync Reset to FADC */
  faSync(FA_SLOT);

  printf("Hit any key to enable Triggers...\n");
  getchar();

  /* Enable the TIR and clear the trigger counter */
  tirIntEnable(TIR_CLEAR_COUNT);

  printf("Hit any key to Disable TIR and exit.\n");
  getchar();

  tirIntDisable();

  tirIntDisconnect();

/* FADC Disable */
  faDisable(FA_SLOT,0);

/* FADC Event status - Is all data read out */
  faStatus(FA_SLOT,0);

  faReset(FA_SLOT,0);
 CLOSE:

  tirIntStatus(1);

  vmeCloseDefaultWindows();

  exit(0);
}

