/*
 * File:
 *    fadcLibTest3.c
 *
 * Description:
 *    Test DMA transfers with a polling thread
 *    with the dmaPoll library
 *
 *
 */
#define _GNU_SOURCE

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include "gef/gefcmn_vme_framework.h"
#include "gef/gefcmn_vme.h"
#include "jlabgef.h"
#include "dmaPollLib.h"
#include "dmaPList.h"
#include "tirLib.h"
#include "fadcLib.h"

#define TIR_MODE TIR_EXT_POLL

// external prototypes (not included in above headers)
GEF_STATUS vmeOpenDefaultWindows();
GEF_STATUS vmeCloseDefaultWindows();

extern int fadcA32Base;

extern unsigned int tirIntCount;
int FA_SLOT;

#define FADC_ADDR 0xee0000

// ISR Mutex
pthread_mutex_t isr_mutex=PTHREAD_MUTEX_INITIALIZER;

/* #define DEBUG */

unsigned long long int rdtsc(void)
{
/*    unsigned long long int x; */
   unsigned a, d;
   
   __asm__ volatile("rdtsc" : "=a" (a), "=d" (d));

   return ((unsigned long long)a) | (((unsigned long long)d) << 32);;
}

// definitions for dmaPList 
#define MAX_NUM_EVENTS    400
#define MAX_SIZE_EVENTS   1024*10      /* Size in Bytes */
#define MAX_DMA_WORDS     1024*10/4       /* Must be less than MAX_SIZE_EVENTS/4 */

/* define some Macros for byte swapping */
#define LSWAP(x)        ((((x) & 0x000000ff) << 24) | \
                         (((x) & 0x0000ff00) <<  8) | \
                         (((x) & 0x00ff0000) >>  8) | \
                         (((x) & 0xff000000) >> 24))

#define SSWAP(x)        ((((x) & 0x00ff) << 8) | \
                         (((x) & 0xff00) >> 8))


unsigned long long starttime=0,endtime=0;
unsigned long long time_now=0,time_prev=0;
double rate=0.; 
double rate_avg=0.;
double dma_rate=0.;
double dma_rate_avg=0.;
extern int fadcID[20];
unsigned short iflag;
int ievent=0;
int nevents=10;
unsigned long long time1=0,time2=0;
unsigned long long clockcalib=0;
DMA_MEM_ID vmeIN,vmeOUT;

int mydefHeader(int);
int mydefNwords(int);
int mydefTrailer(int);
void mytirISR(int arg);

GEF_VME_BUS_HDL vmeHdl;
 
int 
main(int argc, char *argv[]) {

    GEF_STATUS status;
    GEF_UINT32 srcAddr=0x08000000;//, size;//, *lptr;
    int nwords=MAX_DMA_WORDS;
    int stat;
    unsigned short iflag;
    cpu_set_t testCPU;
    

    time1 = rdtsc();
    sleep(1);
    time2 = rdtsc();
    clockcalib=time2-time1;

    if (pthread_getaffinity_np(pthread_self(), sizeof(testCPU), &testCPU) <0) {
      perror("pthread_getaffinity_np");
    }
    printf("f1tdcLibTest2: CPUset = %d\n",testCPU);

    CPU_ZERO(&testCPU);
    CPU_SET(0,&testCPU); // set it to be the same as the dmaPollLib polling thread
    if (pthread_setaffinity_np(pthread_self(),sizeof(testCPU), &testCPU) <0) {
      perror("pthread_setaffinity_np");
    }
    if (pthread_getaffinity_np(pthread_self(), sizeof(testCPU), &testCPU) <0) {
      perror("pthread_getaffinity_np");
    }
    printf("fadcLibTest3: CPUset = %d\n",testCPU);

    if(vmeOpenDefaultWindows() != GEF_STATUS_SUCCESS) {
      goto CLOSE;
    }

    tempeDisableBERRIrq();

    gefVmeSetDebugFlags(vmeHdl,0x0);

    tirIntInit(0,TIR_MODE,1);
    tirIntStatus(1);
  
/*     iflag = 0xea00; // SDC Board address */
    iflag = 0x0; // SDC Board address
    iflag |= 1<<0;  // Front panel sync-reset
    iflag |= 1<<1;  // Front Panel Input trigger source
    iflag |= 0<<4;  // Internal 250MHz Clock source
/*     iflag |= 1<<4;  // Front Panel 250MHz Clock source */
    printf("iflag = 0x%x\n",iflag);

    fadcA32Base = 0x08000000;

    faInit(FADC_ADDR,0x0,1,iflag);
    FA_SLOT = fadcID[0];

    /*      Setup FADC Programming */
    faSetBlockLevel(FA_SLOT,1);
    /*  for Block Reads */
    faEnableBusError(FA_SLOT);
    /*  for Single Cycle Reads */
/*       faDisableBusError(FA_SLOT); */
  
    /*  Set All channel thresholds to 0 */
    faSetThreshold(FA_SLOT,0,0xffff);
  
    
    /*  Setup option 1 processing - RAW Window Data     <-- */
    /*        option 2            - RAW Pulse Data */
    /*        option 3            - Integral Pulse Data */
    /*  Setup 200 nsec latency (PL  = 50)  */
    /*  Setup  80 nsec Window  (PTW = 20) */
    /*  Setup Pulse widths of 36ns (NSB(3)+NSA(6) = 9)  */
    /*  Setup up to 1 pulse processed */
    /*  Setup for both ADC banks(0 - all channels 0-15) */
    faSetProcMode(FA_SLOT,1,313,313,3,6,1,0);
    

    faStatus(0,0);

    faSDC_Config(1,0);

    faSDC_Status(0);
    stat = tirIntConnect(TIR_INT_VEC, mytirISR, 0);
    if (stat != OK) {
      printf("ERROR: tirIntConnect failed: \n");
      goto CLOSE;
    } else {
      printf("INFO: Attached TIR Interrupt\n");
    }


    /*  Enable FADC */
    faEnable(FA_SLOT,0,0);

    taskDelay(1);
  
    /*  Send Sync Reset to FADC */
/*     faSync(FA_SLOT); */
    faSDC_Sync();


    dmaPartInit();
/*     dmaPGetGefVmeHandle(vmeHdl); */

    status = dmaPollInit(fadcA32Base,(MAX_SIZE_EVENTS)>>2,
			 GEF_VME_TRANSFER_MODE_BLT,1);
    if(status != 0) 
      goto CLOSE;

    // Set a bunch of stuff specific to DMA transfers from FADCs
    dmaPollSet(4,GEF_VME_2ESST_RATE_267);
/*     dmaPollSet(6,GEF_VME_TRANSFER_MODE_BLT); */
    dmaPollSet(6,GEF_VME_TRANSFER_MODE_2eSST);
    dmaPollSet(9,GEF_VME_DMA_VOFF_0us | GEF_VME_DMA_VON_4096 | 
	       GEF_VME_DMA_VFAR | GEF_VME_DMA_PFAR |
	       GEF_VME_DMA_NO_INT_ON_BERR);

    startDmaThreads;
    dmaWaitForInit();

/*     dmaDefineHeaderFunction(&mydefHeader); */
/*     dmaDefineNWordsFunction(&mydefNwords,1); */
/*     dmaDefineTrailerFunction(&mydefTrailer); */

    dmaPFreeAll();
    vmeIN  = dmaPCreate("vmeIN",MAX_SIZE_EVENTS,MAX_NUM_EVENTS,0);
    vmeOUT = dmaPCreate("vmeOUT",0,0,0);

    dmaPStatsAll();

    dmaPReInitAll();

    printf("Hit any key to enable Triggers...\n");
    getchar();

    /* Enable the TIR and clear the trigger counter */
    tirIntEnable(TIR_CLEAR_COUNT);

    printf("Hit any key to Disable TIR and exit.\n");
    getchar();

    tirIntDisable();

    tirIntDisconnect();

    /* FADC Disable */
    faDisable(FA_SLOT,0);

    /* FADC Event status - Is all data read out */
    faStatus(FA_SLOT,0);

    faReset(FA_SLOT,0);
    
    
    
    printf("************************************************************\n");
    printf("No more events... stop the threads\n");

    dmaStopThreads();
    
    dmaPFreeAll();
    dmaPStatsAll();
    
    printf("tirIntCount = %d\n",tirIntCount);

 CLOSE:
    tirIntStatus(1);
    
    vmeCloseDefaultWindows();

    printf("average rate = %lf\t average dma rate = %lf\n",rate_avg,dma_rate_avg);

    exit(0);
}

int mydefHeader(int word) {
  if ((LSWAP(word)&FA_DATA_TYPE_DEFINE) && 
      ((LSWAP(word)&FA_DATA_TYPE_MASK) == FA_DATA_BLOCK_HEADER)) return 1;
  else return 0;
}

int mydefNwords(int word) {
  // this isn't used... ever... stupid mistake
  return LSWAP(word)&0xffff;
}

int mydefTrailer(int word) {
  if ((LSWAP(word)&FA_DATA_TYPE_DEFINE) && 
      ((LSWAP(word)&FA_DATA_TYPE_MASK) == FA_DATA_BLOCK_TRAILER)) return 1;
  else return 0;
}

////////////////////////////////////////////////////////////
/* Interrupt Service routine */
////////////////////////////////////////////////////////////

void
mytirISR(int arg)
{
  unsigned short tirdata;
  int ii,stat;
  int status;
  int pollstatus;

  pthread_mutex_lock(&isr_mutex);

#ifdef DEBUG
  printf("event = %d\n",tirIntCount);
#endif
  ievent = tirIntCount;


  if(time_now) {
    time_prev=time_now;
  }
  time_now = rdtsc();
  if(time_prev) {
    rate = 1./(((double)(time_now-time_prev))/(double)clockcalib);
    rate_avg = (rate_avg*(ievent-1) + rate)/(ievent);
  }

  GETEVENT(vmeIN,ievent);

#ifdef DEBUG
  printf("the_event dma_hdl = 0x%x\n",(unsigned int)(the_event->dmaHdl));
  printf("vmeIN dma_hdl = 0x%x\n",(unsigned int)(vmeIN->dmaHdl));
#endif

/*   printf("phys_addr = 0x%x\n",dmaHdl_to_PhysAddr(the_event->dmaHdl)); */

/*   if(tirIntCount==1) dmaPollSetDmaPList(the_event->dmaHdl); */

#ifdef DEBUG
  printf("beginning = 0x%x\t0x%x\n",
	 (unsigned int)dma_dabufp,(unsigned int)&(the_event->length));
  int diff ;
  diff = (int)dma_dabufp-(int)&(the_event->length);
  printf("now = 0x%x \t0x%x \t0x%x\n",
	 (unsigned int)dma_dabufp,(unsigned int)&(the_event->length),diff);
#endif

  // "Event header"
  *dma_dabufp++ = LSWAP(0xd00dd00d);

  // TIR data
  tirdata = tirReadData();
  *dma_dabufp++ = LSWAP(tirdata);


#ifdef DEBUG
  diff = (int)dma_dabufp-(int)&(the_event->data[0]);

  printf("vmeIN->part[0] = 0x%x\t the_event->part = 0x%x\n",
	 (unsigned int)vmeIN->part[0],(unsigned int)&the_event->part);
  printf("now = 0x%x \t0x%x \t0x%x\n",
	 (unsigned int)dma_dabufp,(unsigned int)&(the_event->length),diff);
  printf("data = 0x%x\n",
	 (unsigned int)&(the_event->data[0]));
  printf("the_event dma_hdl = 0x%x\n",(unsigned int)(the_event->dmaHdl));
      
  printf("Start DMA transfer\n");
#endif

  // check for data ready in the fadc
  for(ii=0;ii<100;ii++) {
    stat = faDready(FA_SLOT);
    if (stat>0) {
      break;
    }
  }
  if(stat>0) {
    *dma_dabufp++ = LSWAP(stat);
    starttime=rdtsc();

    pollstatus = faReadBlock(FA_SLOT,dma_dabufp,MAX_DMA_WORDS-sizeof(DMANODE),1);
/*     pollstatus = faReadBlock(FA_SLOT,dma_dabufp,MAX_DMA_WORDS,1); */
/*     dmaPollStartPList(MAX_DMA_WORDS); */
      
/*     pollstatus = dmaPollStatus(); */
    endtime=rdtsc();

/*     jlabgefReadDMARegs(); */

    dma_rate = 1./(((double)(endtime-starttime))/(double)clockcalib);
    if(ievent>0) 
      dma_rate_avg = (dma_rate_avg*(ievent-1) + dma_rate)/(ievent);
#ifdef DEBUG
    printf("main: dma transfer time = %d\n",endtime-starttime);
    printf("pollstatus = %d\n",pollstatus);
#endif

    if(pollstatus < 0) {
      printf("ERROR: in transfer (event = %d), status = 0x%x\n", tirIntCount,pollstatus);
      *dma_dabufp++ = LSWAP(0xda000bad);
    } else {
      dma_dabufp += pollstatus;
    }
  } else {
    printf("ERROR: Data not ready in event %d\n",tirIntCount);
    *dma_dabufp++ = LSWAP(0xda000bad);
  }
  *dma_dabufp++ = LSWAP(0xd00dd00d);

  *dma_dabufp++ = LSWAP(0xcebaf111);
  stat = dmaGetTimes(dma_dabufp);
  dma_dabufp += stat;

  PUTEVENT(vmeOUT);

#ifdef DEBUG
  dmaPStatsAll();
#endif
  int len;
  DMANODE *outEvent;
      
  outEvent = dmaPGetItem(vmeOUT);
#ifdef DEBUG
  printf("outEvent->dmaHdl = 0x%x\n",(unsigned int)(outEvent->dmaHdl));
#endif
  if(outEvent != NULL) {
    len = outEvent->length;
    if(tirIntCount%1000 == 0) {
      printf("Got %d interrupts (TIRdata = 0x%x) (len = %d)\n",tirIntCount,tirdata,len);
      for(ii=0; ii<len; ii++) {
	if((ii%5) == 0) printf("\n    ");
	printf(" 0x%08x ",(unsigned int)LSWAP(outEvent->data[ii]));
      }
      printf("\n\n");
    }
    dmaPFreeItem(outEvent);
  }else{
    logMsg("Error: no Event in vmeOUT queue\n",0,0,0,0,0,0);
  }
#ifdef DEBUG
  printf("\n\n");

  dmaPStatsAll();
  if(time_prev) 
    printf("rate = %lf\n",rate);
  printf("done with event %d\n\n",ievent);
#endif

  tirIntAck();

  pthread_mutex_unlock(&isr_mutex);
}
